/* $Id: code03e8.c,v 1.13 2003/04/02 03:43:10 chrisf Exp $ */

/*
 * PBJTime - Clock utility
 * Copyright (C) 2003 Chris Faherty
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <PalmOS.h>
#include <System/FeatureMgr.h>
#include <System/SysEvtMgr.h>
#include "pbjtimehackRsc.h"
#include "pbjtimehack.h"

/*
 * This is the code that will be called whenever ANY app calls SysHandleEvent,
 * and myhack is enabled in HackMaster. HackMaster is not smart about the
 * content of this file; it will simply jump to the first executable statement
 * in this file. If you need helper functions, prototype them above, and
 * define them AFTER the trap patch.
 */

Boolean MySysHandleEvent(EventPtr eventP)
{
    Boolean (*oldTrap)(EventPtr);
    UInt16 cardNo;
    LocalID dbID;
    DmSearchStateType searchInfo;
    UInt32 crID;
    UInt32 res;
    FormActiveStateType formState;
    Err err;
    UInt32 delayticks;
    Boolean isgood;

    switch (eventP->eType) {

    case keyDownEvent:
        if (eventP->data.keyDown.modifiers&commandKeyMask) {
            if (!is_hotkey(eventP->data.keyDown.chr)) break;

            /* is it already running? */
            err = SysCurAppDatabase(&cardNo, &dbID);
            if (err) return true;
            err = DmDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, 
					NULL, NULL, NULL, NULL, NULL, NULL, NULL, &crID);
            if (err || (crID == 'PBJT')) break;

            /* has key been held down long enough? */
            delayticks = TimGetTicks()+40; /* .4 sec */
            isgood = false;
            while (KeyCurrentState()) {
                if (TimGetTicks() >= delayticks) {
                    isgood = true;
                    break;
                }
            }
            /* key not held down long enough */
            if (!isgood) break;

            /* launch it */
            if ((DmGetNextDatabaseByTypeCreator(true, &searchInfo,
                sysFileTApplication, 'PBJT', true, &cardNo, &dbID) == 0)) {
                /* save all of the current app's UI state */
                FrmSaveActiveState(&formState);
                SysAppLaunch(cardNo, dbID,
                    sysAppLaunchFlagNewStack|sysAppLaunchFlagNewGlobals,
                    sysAppLaunchCmdCustomBase+1, NULL, &res);
                /* restore all of the current app's UI state */
                FrmRestoreActiveState(&formState);
                EvtFlushKeyQueue();
            }

            if (eventP->data.keyDown.modifiers&poweredOnKeyMask) {
                /* power back off */
        		EvtEnqueueKey(vchrPowerOff, vchrPowerOff, commandKeyMask);
            }

            return true;
        }
        break;

    default: break;

    }

    /* get the address of the old SysHandleEvent() from HackMaster */
    FtrGet(MYCRID, 0x3e8, (UInt32 *)&oldTrap);

    return (*oldTrap)(eventP);
}

Boolean is_hotkey(UInt16 chr)
{
    UInt32 tui32;
    UInt16 prefsVersion;
    UInt16 prefsSize;
    UInt16 i;

    /* check the feature first */
    if (FtrGet(MYCRID, FTR_HOTKEY, &tui32)) {
        /* no feature?  then load from prefs */
        i = 0;
        prefsSize = sizeof(i);
        prefsVersion = PrefGetAppPreferences(MYCRID, 0 /* Pref ID */,
            &i, &prefsSize, true);
        tui32 = i;
        /* now save to a feature for efficiency */
        FtrSet(MYCRID, FTR_HOTKEY, tui32);
    }

    return (chr == tui32);
}
